@@ -17,12 +17,16 @@
  
  
 #include "../include/FileSystem.h"
+#include "../include/GParted_Core.h"
 
 #include <cerrno>
+#include <iostream>
+#include <gtkmm/main.h>
+#include <fcntl.h>
 
 namespace GParted
 {
-	
+
 FileSystem::FileSystem()
 {
 }
@@ -48,44 +52,104 @@ const Glib::ustring FileSystem::get_generic_text( CUSTOM_TEXT ttype, int index )
 	}
 }
 
-int FileSystem::execute_command( const Glib::ustring & command, OperationDetail & operationdetail ) 
+void FileSystem::store_exit_status( GPid pid, int status )
 {
-	operationdetail .add_child( OperationDetail( command, STATUS_NONE, FONT_BOLD_ITALIC ) ) ;
-
-	int exit_status = Utils::execute_command( "nice -n 19 " + command, output, error ) ;
-
-	if ( ! output .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( output, STATUS_NONE, FONT_ITALIC ) ) ;
-	
-	if ( ! error .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( error, STATUS_NONE, FONT_ITALIC ) ) ;
-
-	return exit_status ;
+	exit_status = status;
+	running = false;
+	if (pipecount == 0) // pipes finished first
+		Gtk::Main::quit();
+	Glib::spawn_close_pid( pid );
 }
 
-//Time command, add results to operation detail and by default set success or failure
-int FileSystem::execute_command_timed( const Glib::ustring & command
-                                     , OperationDetail & operationdetail
-                                     , bool check_status )
+static void relay_update( OperationDetail *operationdetail, Glib::ustring *str )
 {
-	operationdetail .add_child( OperationDetail( command, STATUS_EXECUTE, FONT_BOLD_ITALIC ) ) ;
+	operationdetail->set_description( *str, FONT_ITALIC );
+}
 
-	int exit_status = Utils::execute_command( "nice -n 19 " + command, output, error ) ;
-	if ( check_status )
-	{
-		if ( ! exit_status )
-			operationdetail .get_last_child() .set_status( STATUS_SUCCES ) ;
-		else
-			operationdetail .get_last_child() .set_status( STATUS_ERROR ) ;
+static void cancel_command( bool force, Glib::Pid pid, bool cancel_safe )
+{
+	if( force || cancel_safe )
+		kill( -pid, SIGINT );
+}
+
+static void setup_child()
+{
+	setpgrp();
+}
+
+int FileSystem::execute_command( const Glib::ustring & command, OperationDetail & operationdetail,
+				 bool checkstatus, bool cancel_safe )
+{
+	operationdetail .add_child( OperationDetail( command, checkstatus ? STATUS_EXECUTE : STATUS_NONE, FONT_BOLD_ITALIC ) ) ;
+	Glib::Pid pid;
+	// set up pipes for capture
+	int out, err;
+	// spawn external process
+	running = true;
+	try {
+		Glib::spawn_async_with_pipes(
+			std::string(),
+			Glib::shell_parse_argv( command ),
+			Glib::SPAWN_DO_NOT_REAP_CHILD | Glib::SPAWN_SEARCH_PATH,
+			sigc::ptr_fun(setup_child),
+			&pid,
+			0,
+			&out,
+			&err );
+	} catch (Glib::SpawnError &e) {
+		std::cerr << e.what() << std::endl;
+		operationdetail.get_last_child().add_child(
+			OperationDetail( e.what(), STATUS_ERROR, FONT_ITALIC ) );
+		return 1;
 	}
+	fcntl( out, F_SETFL, O_NONBLOCK );
+	fcntl( err, F_SETFL, O_NONBLOCK );
+	Glib::signal_child_watch().connect( sigc::mem_fun( *this, &FileSystem::store_exit_status ), pid );
+	output.clear();
+	error.clear();
+	pipecount = 2;
+	PipeCapture outputcapture( out, output );
+	PipeCapture errorcapture( err, error );
+	outputcapture.eof.connect( sigc::mem_fun( *this, &FileSystem::execute_command_eof ) );
+	errorcapture.eof.connect( sigc::mem_fun( *this, &FileSystem::execute_command_eof ) );
+	operationdetail.get_last_child().add_child(
+		OperationDetail( output, STATUS_NONE, FONT_ITALIC ) );
+	operationdetail.get_last_child().add_child(
+		OperationDetail( error, STATUS_NONE, FONT_ITALIC ) );
+	std::vector<OperationDetail> &children = operationdetail.get_last_child().get_childs();
+	outputcapture.update.connect( sigc::bind( sigc::ptr_fun( relay_update ),
+						  &(children[children.size() - 2]),
+						  &output ) );
+	errorcapture.update.connect( sigc::bind( sigc::ptr_fun( relay_update ),
+						 &(children[children.size() - 1]),
+						 &error ) );
+	outputcapture.connect_signal( out );
+	errorcapture.connect_signal( err );
 
-	if ( ! output .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( output, STATUS_NONE, FONT_ITALIC ) ) ;
+	operationdetail.get_last_child().signal_cancel.connect(
+		sigc::bind(
+			sigc::ptr_fun( cancel_command ),
+			pid,
+			cancel_safe ));
+	Gtk::Main::run();
 
-	if ( ! error .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( error, STATUS_NONE, FONT_ITALIC ) ) ;
+	if (checkstatus) {
+		if ( !exit_status )
+			operationdetail.get_last_child().set_status( STATUS_SUCCES );
+		else
+			operationdetail.get_last_child().set_status( STATUS_ERROR );
+	}
+	close( out );
+	close( err );
+	return exit_status;
+}
 
-	return exit_status ;
+void FileSystem::execute_command_eof()
+{
+	if (--pipecount)
+		return; // wait for second pipe to eof
+	if ( !running ) // already got exit status
+		Gtk::Main::quit();
 }
 
 //Create uniquely named temporary directory and add results to operation detail
