@@ -17,16 +17,23 @@
  */
 
 #include "../include/Utils.h"
+#include "../include/GParted_Core.h"
+#include "../include/PipeCapture.h"
 
 #include <sstream>
 #include <fstream>
 #include <iomanip>
+#ifdef HAVE_GLIB_REGEX
 #include <glibmm/regex.h>
+#else
+#include <regex.h>
+#endif
 #include <locale.h>
 #include <uuid/uuid.h>
 #include <cerrno>
 #include <sys/statvfs.h>
-
+#include <gtkmm/main.h>
+#include <fcntl.h>
 
 namespace GParted
 {
@@ -75,12 +82,14 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_UNALLOCATED	: return "#A9A9A9" ;	// ~ medium grey
 		case FS_UNKNOWN		: return "#000000" ;	//black
 		case FS_UNFORMATTED	: return "#000000" ;	//black
+		case FS_CLEARED		: return "#000000" ;	//black
 		case FS_EXTENDED	: return "#7DFCFE" ;	// ~ light blue
 		case FS_BTRFS		: return "#FF9955" ;	//orange
 		case FS_EXT2		: return "#9DB8D2" ;	//blue hilight
 		case FS_EXT3		: return "#7590AE" ;	//blue medium
 		case FS_EXT4		: return "#4B6983" ;	//blue dark
 		case FS_LINUX_SWAP	: return "#C1665A" ;	//red medium
+		case FS_F2FS		: return "#DF421E" ;	//accent red
 		case FS_FAT16		: return "#00FF00" ;	//green
 		case FS_FAT32		: return "#18D918" ;	// ~ medium green
 		case FS_EXFAT		: return "#2E8B57" ;	// ~ sea green
@@ -118,6 +127,59 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::get_color_as_pixbuf( FILESYSTEM filesystem, int
 	return pixbuf ;
 }
 
+int Utils::get_filesystem_label_maxlength( FILESYSTEM filesystem )
+{
+	switch( filesystem )
+	{
+		//All file systems commented out are not supported for labelling
+		//  by either the new partition or label partition operations.
+		case FS_BTRFS		: return 255 ;
+		//case FS_EXFAT		: return  ;
+		case FS_EXT2		: return 16 ;
+		case FS_EXT3		: return 16 ;
+		case FS_EXT4		: return 16 ;
+		//mkfs.f2fs says that it can create file systems with labels up to 512
+		//  characters, but it core dumps with labels of 29 characters or larger!
+		//  Also blkid only correctly displays labels up to 19 characters.
+		//  (Suspect it is all part of a memory corruption bug in mkfs.f2fs).
+		case FS_F2FS		: return 19 ;
+		case FS_FAT16		: return 11 ;
+		case FS_FAT32		: return 11 ;
+		//mkfs.hfsplus can create hfs and hfs+ file systems with labels up to 255
+		//  characters.  However there is no specific tool to read the labels and
+		//  blkid, the only tool currently available, only display the first 27
+		//  and 63 character respectively.
+		//  Reference:
+		//  util-linux-2.20.1/libblkid/src/superblocks/hfs.c:struct hfs_mdb
+		case FS_HFS		: return 27 ;
+		case FS_HFSPLUS		: return 63 ;
+		//mkfs.jfs and jfs_tune can create and update labels to 16 characters but
+		//  only displays the first 11 characters.  This is because version 1 jfs
+		//  file systems only have an 11 character field for the label but version
+		//  2 jfs has extra fields containing a 16 character label.  mkfs.jfs
+		//  writes the extra fields containing the 16 character label, but then
+		//  sets it to version 1 jfs.  It does this to be backwardly compatible
+		//  with jfs before 1.0.18, released May 2002.  Blkid does display the
+		//  full 16 character label by just ignoring the file system version.
+		//  As using jfs_tune to get the label stick with an 11 character limit.
+		//  References:
+		//  jfsutils-1.1.15/tune/tune.c:main()
+		//  jfsutils-1.1.15/mkfs/mkfs.c:create_aggregate()
+		//  http://jfs.cvs.sourceforge.net/viewvc/jfs/jfsutils/NEWS?revision=HEAD
+		case FS_JFS		: return 11 ;
+		case FS_LINUX_SWAP	: return 15 ;
+		//case FS_LVM2_PV	: return  ;
+		case FS_NILFS2		: return 80 ;
+		case FS_NTFS		: return 128 ;
+		case FS_REISER4		: return 16 ;
+		case FS_REISERFS	: return 16 ;
+		//case FS_UFS		: return  ;
+		case FS_XFS		: return 12 ;
+
+		default			: return 30 ;
+	}
+}
+
 Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 {
 	switch( filesystem )
@@ -142,12 +204,19 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 				 * be formatted with a known file system by GParted.
 				 */
 				_("unformatted") ;
+		case FS_CLEARED		: return
+				/* TO TRANSLATORS:  cleared
+				 * means that all file system signatures in the partition
+				 * will be cleared by GParted.
+				 */
+				_("cleared") ;
 		case FS_EXTENDED	: return "extended" ;
 		case FS_BTRFS		: return "btrfs" ;
 		case FS_EXT2		: return "ext2" ;
 		case FS_EXT3		: return "ext3" ;
 		case FS_EXT4		: return "ext4" ;
 		case FS_LINUX_SWAP	: return "linux-swap" ;
+		case FS_F2FS		: return "f2fs" ;
 		case FS_FAT16		: return "fat16" ;
 		case FS_FAT32		: return "fat32" ;
 		case FS_EXFAT		: return "exfat" ;
@@ -177,6 +246,7 @@ Glib::ustring Utils::get_filesystem_software( FILESYSTEM filesystem )
 		case FS_EXT2        : return "e2fsprogs" ;
 		case FS_EXT3        : return "e2fsprogs" ;
 		case FS_EXT4        : return "e2fsprogs v1.41+" ;
+		case FS_F2FS        : return "f2fs-tools" ;
 		case FS_FAT16       : return "dosfstools, mtools" ;
 		case FS_FAT32       : return "dosfstools, mtools" ;
 		case FS_HFS         : return "hfsutils" ;
@@ -221,7 +291,7 @@ bool Utils::kernel_supports_fs( const Glib::ustring & fs )
 		return true ;
 
 	Glib::ustring output, error ;
-	execute_command( "modprobe " + fs, output, error, true ) ;
+	execute_command( "modprobe " + fs, output, error, true );
 
 	input .open( "/proc/filesystems" ) ;
 	if ( input )
@@ -332,62 +402,117 @@ int Utils::execute_command( const Glib::ustring & command )
 	return execute_command( command, dummy, dummy ) ;
 }
 
-int Utils::execute_command( const Glib::ustring & command,
-		     	    Glib::ustring & output,
-			    Glib::ustring & error,
-		     	    bool use_C_locale )
+class utils_execute_command_status
 {
-	int exit_status = -1 ;
-	std::string std_out, std_error ;
+public:
+	bool running;
+	int pipecount;
+	int exit_status;
+	bool foreground;
+	Glib::Mutex mutex;
+	Glib::Cond cond;
+	void store_exit_status( GPid pid, int status );
+	void execute_command_eof();
+};
 
-	try
+void utils_execute_command_status::store_exit_status( GPid pid, int status )
+{
+	exit_status = status;
+	running = false;
+	if (pipecount == 0) // pipes finished first
 	{
-		std::vector<std::string>argv;
-		argv .push_back( "sh" ) ;
-		argv .push_back( "-c" ) ;
-		argv .push_back( command ) ;
-
-		if ( use_C_locale )
-		{
-			//Spawn command using the C language environment
-			std::vector<std::string> envp ;
-			envp .push_back( "LC_ALL=C" ) ;
-			envp .push_back( "PATH=" + Glib::getenv( "PATH" ) ) ;
-
-			Glib::spawn_sync( "."
-			                , argv
-			                , envp
-			                , Glib::SPAWN_SEARCH_PATH
-			                , sigc::slot<void>()
-			                , &std_out
-			                , &std_error
-			                , &exit_status
-			                ) ;
-		}
-		else
-		{
-			//Spawn command inheriting the parent's environment
-			Glib::spawn_sync( "."
-			                , argv
-			                , Glib::SPAWN_SEARCH_PATH
-			                , sigc::slot<void>()
-			                , &std_out
-			                , &std_error
-			                , &exit_status
-			                ) ;
+		if (foreground)
+			Gtk::Main::quit();
+		else {
+			mutex.lock();
+			cond.signal();
+			mutex.unlock();
 		}
 	}
-	catch ( Glib::Exception & e )
+	Glib::spawn_close_pid( pid );
+}
+
+void utils_execute_command_status::execute_command_eof()
+{
+	if (--pipecount)
+		return; // wait for second pipe to eof
+	if ( !running ) // already got exit status
 	{
-		 error = e .what() ;
-
-		 return -1 ;
+		if (foreground)
+			Gtk::Main::quit();
+		else {
+			mutex.lock();
+			cond.signal();
+			mutex.unlock();
+		}
 	}
+}
 
-	output = Utils::cleanup_cursor( std_out ) ;
-	error = std_error ;
+static void set_locale()
+{
+	setenv( "LC_ALL", "C", 1 );
+}
 
-	return exit_status ;
+static void _store_exit_status( GPid pid, gint status, gpointer data )
+{
+	utils_execute_command_status *sp = (utils_execute_command_status *)data;
+	sp->store_exit_status( pid, status );
+}
+
+int Utils::execute_command( const Glib::ustring & command,
+			    Glib::ustring & output,
+			    Glib::ustring & error,
+			    bool use_C_locale )
+{
+	Glib::Pid pid;
+	// set up pipes for capture
+	int out, err;
+	utils_execute_command_status status;
+	// spawn external process
+	status.running = true;
+	status.pipecount = 2;
+	status.foreground = (Glib::Thread::self() == GParted_Core::mainthread);
+	try {
+		Glib::spawn_async_with_pipes(
+			std::string(),
+			Glib::shell_parse_argv( command ),
+			Glib::SPAWN_DO_NOT_REAP_CHILD | Glib::SPAWN_SEARCH_PATH,
+			use_C_locale ? sigc::ptr_fun( set_locale ) : sigc::slot< void >(),
+			&pid,
+			0,
+			&out,
+			&err );
+	} catch (Glib::SpawnError &e) {
+		std::cerr << e.what() << std::endl;
+		return 1;
+	}
+	fcntl( out, F_SETFL, O_NONBLOCK );
+	fcntl( err, F_SETFL, O_NONBLOCK );
+	g_child_watch_add( pid, _store_exit_status, &status );
+	output.clear();
+	error.clear();
+	//Lock mutex so we have time to setup pipecapture for output and error streams
+	//  before connecting the input/output signal handler
+	if( !status.foreground )
+		status.mutex.lock();
+	PipeCapture outputcapture( out, output );
+	PipeCapture errorcapture( err, error );
+	outputcapture.eof.connect( sigc::mem_fun(
+		 status, &utils_execute_command_status::execute_command_eof ));
+	errorcapture.eof.connect( sigc::mem_fun(
+		 status, &utils_execute_command_status::execute_command_eof ));
+	outputcapture.connect_signal( out );
+	errorcapture.connect_signal( err );
+
+	if( status.foreground)
+		Gtk::Main::run();
+	else {
+		status.cond.wait( status.mutex );
+		status.mutex.unlock();
+	}
+	close( out );
+	close( err );
+	return status.exit_status;
 }
 
 Glib::ustring Utils::regexp_label( const Glib::ustring & text
@@ -396,6 +521,7 @@ Glib::ustring Utils::regexp_label( const Glib::ustring & text
 {
 	//Extract text from a regular sub-expression or pattern.
 	//  E.g., "text we don't want (text we want)"
+#ifdef HAVE_GLIB_REGEX
 	std::vector<Glib::ustring> results;
 	Glib::RefPtr<Glib::Regex> myregexp =
 		Glib::Regex::create( pattern
@@ -408,72 +534,20 @@ Glib::ustring Utils::regexp_label( const Glib::ustring & text
 		return results[ 1 ] ;
 	else
 		return "" ;
-}
-
-Glib::ustring Utils::fat_compliant_label( const Glib::ustring & label )
-{
-	//Limit volume label to 11 characters
-	Glib::ustring text = label ;
-	if( text .length() > 11 )
-		text .resize( 11 ) ;
-	return text ;
-}
-
-Glib::ustring Utils::create_mtoolsrc_file( char file_name[], const char drive_letter,
-		const Glib::ustring & device_path )
-{
-	//Create mtools config file
-	//NOTE:  file_name will be changed by the mkstemp() function call.
-	Glib::ustring err_msg = "" ;
-	int fd ;
-	fd = mkstemp( file_name ) ;
-	if( fd != -1 ) {
-		Glib::ustring fcontents =
-			/* TO TRANSLATORS:  # Temporary file created by gparted.  It may be deleted.
-			 * means that this file is only used while gparted is applying operations.
-			 * If for some reason this file exists at any other time, then the message is
-			 * meant to inform a user that the file can be deleted with no harmful effects.
-			 * This file is typically created, exists for less than a few seconds, and is
-			 * then deleted by gparted.  Under normal circumstances a user should never
-			 * see this file.
-			 */
-			_("# Temporary file created by gparted.  It may be deleted.\n") ;
-
-		//The following file contents are mtools keywords (see man mtools.conf)
-		fcontents = String::ucompose(
-						"drive %1: file=\"%2\"\nmtools_skip_check=1\n", drive_letter, device_path
-					);
-
-		if( write( fd, fcontents .c_str(), fcontents .size() ) == -1 ) {
-			err_msg = String::ucompose(
-							/* TO TRANSLATORS: looks like
-							 * Label operation failed:  Unable to write to temporary file /tmp/Y56ZZ3M13LM.
-							 */
-							_("Label operation failed:  Unable to write to temporary file %1.\n")
-							, file_name
-						) ;
-		}
-		close( fd ) ;
-	}
-	else
+#else /* ! HAVE_GLIB_REGEX */
+	Glib::ustring label = "" ;
+	regex_t preg ;
+	int nmatch = 2 ;
+	regmatch_t pmatch[ 2 ] ;
+	int rc = regcomp( &preg, pattern .c_str(), REG_EXTENDED | REG_ICASE | REG_NEWLINE ) ;
+	if ( rc == 0 )
 	{
-		err_msg = String::ucompose(
-						/* TO TRANSLATORS: looks like
-						 * Label operation failed:  Unable to create temporary file /tmp/Y56ZZ3M13LM.
-						 */
-						_("Label operation failed:  Unable to create temporary file %1.\n")
-						, file_name
-					) ;
+		if ( regexec( &preg, text .c_str(), nmatch, pmatch, 0 ) == 0 )
+			label = text .substr( pmatch[ 1 ] .rm_so, pmatch[ 1 ] .rm_eo - pmatch[ 1 ] .rm_so ) ;
+		regfree( &preg ) ;
 	}
-	return err_msg ;
-}
-
-Glib::ustring Utils::delete_mtoolsrc_file( const char file_name[] )
-{
-	//Delete mtools config file
-	Glib::ustring err_msg = "" ;
-	remove( file_name ) ;
-	return err_msg ;
+	return label ;
+#endif
 }
 
 Glib::ustring Utils::trim( const Glib::ustring & src, const Glib::ustring & c /* = " \t\r\n" */ )
@@ -486,37 +560,6 @@ Glib::ustring Utils::trim( const Glib::ustring & src, const Glib::ustring & c /*
 	return src.substr(p1, (p2-p1)+1);
 }
 
-Glib::ustring Utils::cleanup_cursor( const Glib::ustring & text )
-{
-	std::istringstream in(text);
-	std::ostringstream out;
-	char ch;
-	std::streampos startofline = out.tellp();
-
-	while (in.get(ch))
-	{
-		switch(ch)
-		{
-			case '\r':
-				if ('\n' != in.peek()) // for windows CRLF
-					out.seekp(startofline);
-				else
-					out.put(ch);
-				break;
-			case '\b':
-				if (out.tellp() > startofline)
-					out.seekp(out.tellp() - std::streamoff(1));
-				break;
-			default:
-				out.put(ch);
-		}
-		if (ch == '\n')
-		 	startofline = out.tellp();
-	}
-
-	return out.str();
-}
-
 Glib::ustring Utils::get_lang()
 {
 	//Extract base language from string that may look like "en_CA.UTF-8"
