@@ -1,5 +1,5 @@
 /* Copyright (C) 2004 Bart
- * Copyright (C) 2008, 2009, 2010, 2011, 2012 Curtis Gedak
+ * Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013 Curtis Gedak
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,7 +53,6 @@ Win_GParted::Win_GParted( const std::vector<Glib::ustring> & user_devices )
 	selected_partition .Reset() ;
 	new_count = 1;
 	current_device = 0 ;
-	pulse = false ; 
 	OPERATIONSLIST_OPEN = true ;
 	gparted_core .set_user_devices( user_devices ) ;
 	
@@ -76,14 +75,16 @@ Win_GParted::Win_GParted( const std::vector<Glib::ustring> & user_devices )
 	this ->set_title( _("GParted") );
 	this ->set_default_size( 775, 500 );
 	
+#ifdef HAVE_SET_DEFAULT_ICON_NAME
 	try
 	{
 		this ->set_default_icon_name( "gparted" ) ;
 	}
 	catch ( Glib::Exception & e )
-	{ 
+	{
 		std::cout << e .what() << std::endl ;
-	} 
+	}
+#endif
 	
 	//Pack the main box
 	this ->add( vbox_main ); 
@@ -167,7 +168,8 @@ void Win_GParted::init_menubar()
 		sigc::mem_fun(*this, &Win_GParted::clear_operationslist) ) );
 
 	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( 
-		_("_Apply All Operations"), 
+		_("_Apply All Operations"),
+		Gtk::AccelKey(GDK_Return, Gdk::CONTROL_MASK),
 		* manage( new Gtk::Image( Gtk::Stock::APPLY, Gtk::ICON_SIZE_MENU ) ), 
 		sigc::mem_fun(*this, &Win_GParted::activate_apply) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Edit"), *menu ) );
@@ -310,8 +312,9 @@ void Win_GParted::init_partition_menu()
 	image = manage( new Gtk::Image( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU ) );
 	menu_partition .items() .push_back( 
 			/*TO TRANSLATORS: "_New" is a sub menu item for the partition menu. */
-			Gtk::Menu_Helpers::ImageMenuElem( _("_New"), 
-							  *image, 
+			Gtk::Menu_Helpers::ImageMenuElem( _("_New"),
+							  Gtk::AccelKey( GDK_Insert, Gdk::BUTTON1_MASK),
+							  *image,
 							  sigc::mem_fun(*this, &Win_GParted::activate_new) ) );
 	MENU_NEW = index++ ;
 	
@@ -403,45 +406,49 @@ void Win_GParted::init_partition_menu()
 	menu_partition .accelerate( *this ) ;  
 }
 
+//Create the Partition --> Format to --> (file system list) menu
 Gtk::Menu * Win_GParted::create_format_menu()
 {
+	const std::vector<FS> & fss = gparted_core .get_filesystems() ;
 	menu = manage( new Gtk::Menu() ) ;
 
-	for ( unsigned int t =0; t < gparted_core .get_filesystems() .size() ; t++ )
+	for ( unsigned int t = 0 ; t < fss .size() ; t++ )
 	{
-		//Skip luks and unknown because these are not file systems
-		if (
-		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LUKS    ||
-		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_UNKNOWN
+		//Skip unknown, cleared and luks because these are not file systems
+		if (    fss[ t ] .filesystem == FS_UNKNOWN
+		     || fss[ t ] .filesystem == FS_CLEARED
+		     || fss[ t ] .filesystem == FS_LUKS
 		   )
 			continue ;
 
-		hbox = manage( new Gtk::HBox() );
-			
-		//the colored square
-		hbox ->pack_start( * manage( new Gtk::Image(
-					Utils::get_color_as_pixbuf( 
-						gparted_core .get_filesystems()[ t ] .filesystem, 16, 16 ) ) ),
-				   Gtk::PACK_SHRINK ) ;			
-		
-		//the label...
-		hbox ->pack_start( * Utils::mk_label(
-					" " + 
-					Utils::get_filesystem_string( gparted_core .get_filesystems()[ t ] .filesystem ) ),
-				   Gtk::PACK_SHRINK );	
-				
-		menu ->items() .push_back( * manage( new Gtk::MenuItem( *hbox ) ) );
-		if ( gparted_core .get_filesystems()[ t ] .create )
-			menu ->items() .back() .signal_activate() .connect( 
-				sigc::bind<GParted::FILESYSTEM>(sigc::mem_fun(*this, &Win_GParted::activate_format),
-				gparted_core .get_filesystems()[ t ] .filesystem ) ) ;
-		else
-			menu ->items() .back() .set_sensitive( false ) ;
+		create_format_menu_add_item( fss[ t ] .filesystem, fss[ t ] .create ) ;
 	}
-	
+	//Add cleared at the end of the list
+	create_format_menu_add_item( FS_CLEARED, true ) ;
+
 	return menu ;
 }
-	
+
+//Add one entry to the Partition --> Format to --> (file system list) menu
+void Win_GParted::create_format_menu_add_item( FILESYSTEM filesystem, bool activate )
+{
+	hbox = manage( new Gtk::HBox() ) ;
+	//the colored square
+	hbox ->pack_start( * manage( new Gtk::Image( Utils::get_color_as_pixbuf( filesystem, 16, 16 ) ) ),
+	                   Gtk::PACK_SHRINK ) ;
+	//the label...
+	hbox ->pack_start( * Utils::mk_label( " " + Utils::get_filesystem_string( filesystem ) ),
+	                   Gtk::PACK_SHRINK ) ;
+
+	menu ->items() .push_back( * manage( new Gtk::MenuItem( *hbox ) ) ) ;
+	if ( activate )
+		menu ->items() .back() .signal_activate() .connect(
+			sigc::bind<GParted::FILESYSTEM>( sigc::mem_fun( *this, &Win_GParted::activate_format ),
+			                                 filesystem ) ) ;
+	else
+		menu ->items() .back() .set_sensitive( false ) ;
+}
+
 void Win_GParted::init_device_info()
 {
 	vbox_info.set_spacing( 5 );
@@ -608,6 +615,18 @@ void Win_GParted::refresh_combo_devices()
 	combo_devices .set_active( current_device ) ;
 }
 
+bool Win_GParted::pulsebar_pulse()
+{
+	pulsebar.pulse();
+	Glib::ustring tmp_msg = gparted_core .get_thread_status_message() ;
+	if ( tmp_msg != "" ) {
+		statusbar.pop();
+		statusbar.push( tmp_msg );
+	}
+
+	return true;
+}
+
 void Win_GParted::show_pulsebar( const Glib::ustring & status_message ) 
 {
 	pulsebar .show();
@@ -621,22 +640,13 @@ void Win_GParted::show_pulsebar( const Glib::ustring & status_message )
 	treeview_detail .set_sensitive( false ) ;
 	drawingarea_visualdisk .set_sensitive( false ) ;
 		
-	//the actual 'pulsing'
-	while ( pulse )
-	{
-		pulsebar .pulse();
-		while ( Gtk::Main::events_pending() )
-			Gtk::Main::iteration();
-		gdk_threads_leave();
-		usleep( 100000 );
-		gdk_threads_enter();
-		Glib::ustring tmp_msg = gparted_core .get_thread_status_message() ;
-		if ( tmp_msg != "" )
-			statusbar .push( tmp_msg ) ;
-	}
-	
-	thread ->join() ;
-	
+	// connect pulse update timer
+	pulsetimer = Glib::signal_timeout().connect( sigc::mem_fun(*this, &Win_GParted::pulsebar_pulse), 100 );
+}
+
+void Win_GParted::hide_pulsebar()
+{
+	pulsetimer.disconnect();
 	pulsebar .hide();
 	statusbar .pop() ;
 		
@@ -837,10 +847,17 @@ void Win_GParted::Refresh_Visual()
 		
 	if ( ! operations .size() ) 
 		allow_undo_clear_apply( false ) ;
-			
-	//count primary's and check for extended
+
+	//Count primary partitions for check in max_amount_prim_reached(),
+	//  check for an extended partition and select the largest unallocated
+	//  partition if there is one.
 	index_extended = -1 ;
 	primary_count = 0;
+
+	selected_partition .Reset() ;
+	Sector largest_unalloc_size = -1 ;
+	Sector current_size ;
+
 	for ( unsigned int t = 0 ; t < partitions .size() ; t++ )
 	{
 		if ( partitions[ t ] .get_path() == copied_partition .get_path() )
@@ -848,15 +865,42 @@ void Win_GParted::Refresh_Visual()
 		
 		switch ( partitions[ t ] .type )
 		{
-			case GParted::TYPE_PRIMARY	:
+			case TYPE_PRIMARY:
 				primary_count++;
 				break;
-				
-			case GParted::TYPE_EXTENDED	:
+
+			case TYPE_EXTENDED:
 				index_extended = t ;
 				primary_count++;
+
+				for ( unsigned int u = 0 ; u < partitions[ t ] .logicals .size() ; u ++ )
+				{
+					switch ( partitions[ t ] .logicals[ u ] .type )
+					{
+						case TYPE_UNALLOCATED:
+							current_size = partitions[ t ]. logicals[ u ] .get_sector_length() ;
+							if ( current_size > largest_unalloc_size )
+							{
+								largest_unalloc_size = current_size ;
+								selected_partition = partitions[ t ] .logicals[ u ] ;
+							}
+							break;
+
+						default:
+							break;
+					}
+				}
 				break;
-				
+
+			case TYPE_UNALLOCATED:
+				current_size = partitions[ t ] .get_sector_length() ;
+				if ( current_size > largest_unalloc_size )
+				{
+					largest_unalloc_size = current_size ;
+					selected_partition = partitions[ t ] ;
+				}
+				break;
+
 			default				:
 				break;
 		}
@@ -867,11 +911,16 @@ void Win_GParted::Refresh_Visual()
 
 	//treeview details
 	treeview_detail .load_partitions( partitions ) ;
-	
-	//no partition can be selected after a refresh..
-	selected_partition .Reset() ;
-	set_valid_operations() ; 
-			
+
+	set_valid_operations() ;
+
+	if ( largest_unalloc_size >= 0 )
+	{
+		//Inform visuals of selection of the largest unallocated partition
+		drawingarea_visualdisk .set_selected( selected_partition ) ;
+		treeview_detail .set_selected( selected_partition ) ;
+	}
+
 	while ( Gtk::Main::events_pending() ) 
 		Gtk::Main::iteration() ;
 }
@@ -1198,18 +1247,11 @@ void Win_GParted::on_show()
 	menu_gparted_refresh_devices() ;
 }
 	
-void Win_GParted::thread_refresh_devices() 
-{
-	gparted_core .set_devices( devices ) ;
-	pulse = false ;
-}
-
 void Win_GParted::menu_gparted_refresh_devices()
 {
-	pulse = true ;	
-	thread = Glib::Thread::create( sigc::mem_fun( *this, &Win_GParted::thread_refresh_devices ), true ) ;
-
 	show_pulsebar( _("Scanning all devices...") ) ;
+	gparted_core.set_devices( devices );
+	hide_pulsebar();
 	
 	//check if current_device is still available (think about hotpluggable stuff like usbdevices)
 	if ( current_device >= devices .size() )
@@ -1426,11 +1468,12 @@ void Win_GParted::menu_help_about()
 	dialog .set_logo_icon_name( "gparted" ) ;
 	dialog .set_version( VERSION ) ;
 	dialog .set_comments( _( "GNOME Partition Editor" ) ) ;
-	dialog .set_copyright( "Copyright © 2004-2006 Bart Hakvoort\nCopyright © 2008-2012 Curtis Gedak" ) ;
+	dialog .set_copyright( "Copyright © 2004-2006 Bart Hakvoort\nCopyright © 2008-2013 Curtis Gedak" ) ;
 
 	//authors
 	//Names listed in alphabetical order by LAST name.
 	//See also AUTHORS file -- names listed in opposite order to try to be fair.
+	strings .push_back( "Sinlu Bes <e80f00@gmail.com>" ) ;
 	strings .push_back( "Luca Bruno <lucab@debian.org>" ) ;
 	strings .push_back( "Jérôme Dumesnil <jerome.dumesnil@gmail.com>" ) ;
 	strings .push_back( "Markus Elfring <elfring@users.sourceforge.net>" ) ;
@@ -1980,7 +2023,7 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 	}
 }
 
-void Win_GParted::thread_unmount_partition( bool * succes, Glib::ustring * error ) 
+void Win_GParted::unmount_partition( bool * succes, Glib::ustring * error ) 
 {
 	std::vector<Glib::ustring> errors, failed_mountpoints, mountpoints = gparted_core .get_all_mountpoints() ;
 	Glib::ustring dummy ;
@@ -2012,66 +2055,15 @@ void Win_GParted::thread_unmount_partition( bool * succes, Glib::ustring * error
 	}
 	else
 		*error = "<i>" + Glib::build_path( "\n", errors ) + "</i>" ;
-
-	pulse = false ;
 }
 	
-void Win_GParted::thread_mount_partition( Glib::ustring mountpoint, bool * succes, Glib::ustring * error ) 
-{
-	Glib::ustring dummy ;
-	std::vector<Glib::ustring> errors ;
-	
-	*succes = ! Utils::execute_command( "mount -v " + selected_partition .get_path() + " \"" + mountpoint + "\"",
-					    dummy,
-					    *error ) ;
-
-	pulse = false ;
-}
-
-void Win_GParted::thread_toggle_swap( bool * succes, Glib::ustring * error ) 
-{	
-	Glib::ustring dummy ;
-	
-	if ( selected_partition .busy )
-		*succes = ! Utils::execute_command( "swapoff -v " + selected_partition .get_path() + " && sync",
-						    dummy,
-						    *error ) ;
-	else
-		*succes = ! Utils::execute_command( "swapon -v " + selected_partition .get_path() + " && sync",
-						    dummy,
-						    *error ) ;
-
-	pulse = false ;
-}
-
-void Win_GParted::thread_toggle_lvm2_pv( bool * success, Glib::ustring * error )
-{
-	Glib::ustring dummy ;
-
-	if ( selected_partition .busy )
-		//VGNAME from mount point
-		*success = ! Utils::execute_command( "lvm vgchange -a n " + selected_partition .get_mountpoint(),
-		                                     dummy,
-		                                     *error ) ;
-	else
-		*success = ! Utils::execute_command( "lvm vgchange -a y " + selected_partition .get_mountpoint(),
-		                                     dummy,
-		                                     *error ) ;
-
-	pulse = false ;
-}
-
-// Runs gpart in a thread
-void Win_GParted::thread_guess_partition_table()
-{
-	this->gpart_output="";
-	this->gparted_core.guess_partition_table(devices[ current_device ], this->gpart_output);
-	pulse=false;
-}
-
 void Win_GParted::toggle_busy_state()
 {
 	int operation_count = partition_in_operation_queue_count( selected_partition ) ;
+	bool success = false ;
+	Glib::ustring error ;
+	Glib::ustring output;
+
 	if ( operation_count > 0 )
 	{
 		//Note that this situation will only occur when trying to swapon a partition
@@ -2114,22 +2106,22 @@ void Win_GParted::toggle_busy_state()
 		return ;
 	}
 
-	bool succes = false ;
-	Glib::ustring error ;
-
-	pulse = true ;
-
 	if ( selected_partition .filesystem == GParted::FS_LINUX_SWAP )
 	{
-		thread = Glib::Thread::create( sigc::bind<bool *, Glib::ustring *>( 
-			sigc::mem_fun( *this, &Win_GParted::thread_toggle_swap ), &succes, &error ), true ) ;
-
 		show_pulsebar( 
 			String::ucompose( 
 				selected_partition .busy ? _("Deactivating swap on %1") : _("Activating swap on %1"),
 				selected_partition .get_path() ) ) ;
-
-		if ( ! succes )
+		if ( selected_partition .busy )
+			success = ! Utils::execute_command( "swapoff -v " + selected_partition .get_path(),
+							    output,
+							    error );
+		else
+			success = ! Utils::execute_command( "swapon -v " + selected_partition .get_path(),
+							    output,
+							    error );
+		hide_pulsebar();
+		if ( ! success )
 		{
 			Gtk::MessageDialog dialog( 
 				*this,
@@ -2146,17 +2138,24 @@ void Win_GParted::toggle_busy_state()
 	}
 	else if ( selected_partition .filesystem == GParted::FS_LVM2_PV )
 	{
-		thread = Glib::Thread::create( sigc::bind<bool *, Glib::ustring *>(
-			sigc::mem_fun( *this, &Win_GParted::thread_toggle_lvm2_pv ), &succes, &error ), true ) ;
-
 		show_pulsebar(
 			String::ucompose(
 				selected_partition .busy ? _("Deactivating Volume Group %1")
 				                         : _("Activating Volume Group %1"),
 				//VGNAME from mount point
 				selected_partition .get_mountpoint() ) ) ;
+		if ( selected_partition .busy )
+			//VGNAME from mount point
+			success = ! Utils::execute_command( "lvm vgchange -a n " + selected_partition .get_mountpoint(),
+							    output,
+							    error );
+		else
+			success = ! Utils::execute_command( "lvm vgchange -a y " + selected_partition .get_mountpoint(),
+							     output,
+							     error );
+		hide_pulsebar();
 
-		if ( ! succes )
+		if ( ! success )
 		{
 			Gtk::MessageDialog dialog(
 				*this,
@@ -2174,12 +2173,10 @@ void Win_GParted::toggle_busy_state()
 	}
 	else if ( selected_partition .busy )
 	{
-		thread = Glib::Thread::create( sigc::bind<bool *, Glib::ustring *>( 
-			sigc::mem_fun( *this, &Win_GParted::thread_unmount_partition ), &succes, &error ), true ) ;
-
 		show_pulsebar( String::ucompose( _("Unmounting %1"), selected_partition .get_path() ) ) ;
-	
-		if ( ! succes )
+		unmount_partition( &success, &error );
+		hide_pulsebar();
+		if ( ! success )
 		{
 			Gtk::MessageDialog dialog( *this, 
 						   String::ucompose( _("Could not unmount %1"), selected_partition .get_path() ),
@@ -2226,23 +2223,19 @@ void Win_GParted::activate_mount_partition( unsigned int index )
 		return ;
 	}
 
-	bool succes = false ;
+	bool success = false ;
 	Glib::ustring error ;
-
-	pulse = true ;
-
-	thread = Glib::Thread::create( sigc::bind<Glib::ustring, bool *, Glib::ustring *>( 
-						sigc::mem_fun( *this, &Win_GParted::thread_mount_partition ),
-						selected_partition .get_mountpoints()[ index ],
-						&succes,
-						&error ),
-				       true ) ;
+	Glib::ustring stdout;
 
 	show_pulsebar( String::ucompose( _("mounting %1 on %2"),
 					 selected_partition .get_path(),
 					 selected_partition .get_mountpoints()[ index ] ) ) ;
-
-	if ( ! succes )
+	success = !Utils::execute_command( "mount -v " + selected_partition .get_path() + " \"" +
+					   selected_partition.get_mountpoints()[ index ] + "\"",
+					   stdout,
+					   error ) ;
+	hide_pulsebar();
+	if ( ! success )
 	{
 		Gtk::MessageDialog dialog( *this, 
 					   String::ucompose( _("Could not mount %1 on %2"),
@@ -2381,12 +2374,11 @@ void Win_GParted::activate_attempt_rescue_data()
 
 	messageDialog.hide();
 
-	pulse=true;
-	this->thread = Glib::Thread::create( sigc::mem_fun( *this, &Win_GParted::thread_guess_partition_table ), true ) ;
-
 	/*TO TRANSLATORS: looks like	Searching for file systems on /deb/sdb */
 	show_pulsebar(String::ucompose( _("Searching for file systems on %1"), devices[ current_device ] .get_path()));
-
+	gpart_output="";
+	gparted_core.guess_partition_table(devices[ current_device ], gpart_output);
+	hide_pulsebar();
 	Dialog_Rescue_Data dialog;
 	dialog .set_transient_for( *this );
 
