@@ -68,7 +68,7 @@ FS jfs::get_filesystem_support()
 
 void jfs::set_used_sectors( Partition & partition ) 
 {
-	if ( ! Utils::execute_command( "echo dm | jfs_debugfs " + partition .get_path(), output, error, true ) )
+	if ( ! Utils::execute_command( "sh -c 'echo dm | jfs_debugfs " + partition.get_path() + "'", output, error, true ) )
 	{
 		//blocksize
 		index = output .find( "Block Size:" ) ;
@@ -149,7 +149,9 @@ bool jfs::write_uuid( const Partition & partition, OperationDetail & operationde
 
 bool jfs::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "mkfs.jfs -q -L \"" + new_partition .get_label() + "\" " + new_partition .get_path(), operationdetail ) ;
+	return ! execute_command( "mkfs.jfs -q -L \"" + new_partition.get_label() +
+				  "\" " + new_partition.get_path(), operationdetail,
+				  false, true );
 }
 
 bool jfs::resize( const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition )
@@ -160,15 +162,15 @@ bool jfs::resize( const Partition & partition_new, OperationDetail & operationde
 	if ( mount_point .empty() )
 		return false ;
 
-	success &= ! execute_command_timed( "mount -v -t jfs " + partition_new .get_path() + " " + mount_point,
-	                                    operationdetail ) ;
+	success &= ! execute_command( "mount -v -t jfs " + partition_new .get_path() + " " + mount_point,
+				      operationdetail, true ) ;
 
 	if ( success )
 	{
-		success &= ! execute_command_timed( "mount -v -t jfs -o remount,resize " + partition_new .get_path() + " " + mount_point,
-		                                    operationdetail ) ;
+		success &= ! execute_command( "mount -v -t jfs -o remount,resize " + partition_new .get_path() + " " + mount_point,
+					      operationdetail, true ) ;
 
-		success &= ! execute_command_timed( "umount -v " + mount_point, operationdetail ) ;
+		success &= ! execute_command( "umount -v " + mount_point, operationdetail, true ) ;
 	}
 
 	rm_temp_dir( mount_point, operationdetail ) ;
@@ -176,33 +178,14 @@ bool jfs::resize( const Partition & partition_new, OperationDetail & operationde
 	return success ;
 }
 
-bool jfs::move( const Partition & partition_new
-              , const Partition & partition_old
-              , OperationDetail & operationdetail
-              )
-{
-	return true ;
-}
-
-bool jfs::copy( const Glib::ustring & src_part_path, 
-		const Glib::ustring & dest_part_path,
-		OperationDetail & operationdetail )
-{
-	return true ;
-}
-
 bool jfs::check_repair( const Partition & partition, OperationDetail & operationdetail )
 {
-	exit_status = execute_command( "jfs_fsck -f " + partition .get_path(), operationdetail ) ;
+	exit_status = execute_command( "jfs_fsck -f " + partition.get_path(), operationdetail,
+				       false, true );
 
 	return ( exit_status == 0 || exit_status == 1 ) ;
 }
 
-bool jfs::remove( const Partition & partition, OperationDetail & operationdetail )
-{
-	return true ;
-}
-
 } //GParted
 
 
