@@ -110,8 +110,9 @@ FS ntfs::get_filesystem_support()
 
 void ntfs::set_used_sectors( Partition & partition ) 
 {
-	if ( ! Utils::execute_command( 
-		"ntfsresize --info --force --no-progress-bar " + partition .get_path(), output, error, true ) )
+	exit_status = Utils::execute_command(
+		"ntfsresize --info --force --no-progress-bar " + partition .get_path(), output, error, true ) ;
+	if ( exit_status == 0 || exit_status == 1<<8 )
 	{
 		index = output .find( "Current volume size:" ) ;
 		if ( index >= output .length() ||
@@ -122,6 +123,11 @@ void ntfs::set_used_sectors( Partition & partition )
 		if ( index >= output .length() ||
 		     sscanf( output .substr( index ) .c_str(), "resize at %Ld", &N ) != 1 )
 			N = -1 ;
+		//For an absolutely full NTFS, "ntfsresize --info" exits
+		//  with status 1 and reports this message instead
+		index = output .find( "ERROR: Volume is full" ) ;
+		if ( index < output .length() )
+			N = T ;
 
 		if ( T > -1 && N > -1 )
 		{
@@ -177,7 +183,11 @@ bool ntfs::write_uuid( const Partition & partition, OperationDetail & operationd
 
 bool ntfs::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "mkntfs -Q -v -L \"" + new_partition .get_label() + "\" " + new_partition .get_path(), operationdetail ) ;
+	return ! execute_command( "mkntfs -Q -v -L \"" + new_partition.get_label() +
+				  "\" " + new_partition.get_path(),
+				  operationdetail,
+				  false,
+				  true );
 }
 
 bool ntfs::resize( const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition )
@@ -220,19 +230,14 @@ bool ntfs::resize( const Partition & partition_new, OperationDetail & operationd
 	return return_value ;
 }
 
-bool ntfs::move( const Partition & partition_new
-               , const Partition & partition_old
-               , OperationDetail & operationdetail
-               )
-{
-	return true ;
-}
-
-bool ntfs::copy( const Glib::ustring & src_part_path,
-		 const Glib::ustring & dest_part_path, 
+bool ntfs::copy( const Partition & src_part,
+		 Partition & dest_part, 
 		 OperationDetail & operationdetail )
 {
-	return ! execute_command( "ntfsclone -f --overwrite " + dest_part_path + " " + src_part_path, operationdetail ) ;
+	return ! execute_command( "ntfsclone -f --overwrite " + dest_part.get_path() + " " + src_part.get_path(),
+				  operationdetail,
+				  false,
+				  true );
 }
 
 bool ntfs::check_repair( const Partition & partition, OperationDetail & operationdetail )
@@ -240,11 +245,6 @@ bool ntfs::check_repair( const Partition & partition, OperationDetail & operatio
 	return ! execute_command( "ntfsresize -P -i -f -v " + partition .get_path(), operationdetail ) ; 
 }
 
-bool ntfs::remove( const Partition & partition, OperationDetail & operationdetail )
-{
-	return true ;
-}
-
 } //GParted
 
 
